@@ -4,10 +4,16 @@
 #ifndef NORMALIZE_DATA
 #define NORMALIZE_DATA
 
-#include <math.h>
+#include <cmath>
 
 #include "dataSource.h"
 
+#define GD_RCPP
+
+#ifdef GD_RCPP
+#include <Rcpp.h>
+#endif
+
 using namespace std;
 
 class NormalizeData {
@@ -32,6 +38,10 @@ public:
             float fMin = numeric_limits<float>::max();
             for(int i = 0; i < pNumberColumn->getValueVector().size(); i++) {
                 float number = pNumberColumn->getValueVector()[i];
+                if(isnan(number)) {
+                    continue;    
+                }
+                
                 if(number > fMax) {
                     fMax = number;
                 }
@@ -40,29 +50,67 @@ public:
                 }
             }
             if(calculateMinMax) {
+                if(fMax == numeric_limits<float>::min()) {
+                    fMax = 1.0;
+                }
+                if(fMin == numeric_limits<float>::max()) {
+                    fMin = 0.0;
+                }
                 pNumberColumn->setMax(fMax);
                 pNumberColumn->setMin(fMin);
             }
-        
+            
             pNumberColumn->getNormalizedValueVector().resize(pNumberColumn->getValueVector().size(), 0);
             for(int j = 0; j < (int)pNumberColumn->getValueVector().size(); j++) {
+                if(isnan(pNumberColumn->getValueVector()[j])) {
+                    pNumberColumn->getNormalizedValueVector()[j] = pNumberColumn->getValueVector()[j];
+                    continue;    
+                }
+                
                 float normalizedNumber = 0;
-                if(pNumberColumn->getMax() - pNumberColumn->getMin() > 0) {
-                    if(scaleType == NumberColumn::LINEAR) {
+                if(scaleType == NumberColumn::LINEAR) {
+                    if(pNumberColumn->getMax() - pNumberColumn->getMin() > 0) {
                         normalizedNumber = (pNumberColumn->getValueVector()[j] - pNumberColumn->getMin()) /
                             (pNumberColumn->getMax() - pNumberColumn->getMin());
                     } else {
-                        throw cInvalidScaleType;
+                        if(pNumberColumn->getMax() > 0) {
+                            //normalizedNumber = pNumberColumn->getMax();
+                            normalizedNumber = 1;
+                        }
+                    }
+                } else if(scaleType == NumberColumn::LOGARITHMIC) {
+                    if(pNumberColumn->getMax() - pNumberColumn->getMin() > 0) {
+                        normalizedNumber = log(pNumberColumn->getValueVector()[j] - pNumberColumn->getMin() + 1) /
+                            log(pNumberColumn->getMax() - pNumberColumn->getMin() + 1);
+                    } else {
+                        if(pNumberColumn->getMax() > 0) {
+                            //normalizedNumber = pNumberColumn->getMax();
+                            normalizedNumber = 1;
+                        }
                     }
                 } else {
-                    normalizedNumber = pNumberColumn->getMax();
+                    throw cInvalidScaleType;
                 }
                 pNumberColumn->getNormalizedValueVector()[j] = normalizedNumber;
             }
         } else if(type == Column::STRING) {
-            throw string(cInvalidColumnType);
+            string invalidColumnType = cInvalidColumnPrefix + " ";
+            for(int i = 0; i < pColumn->getName().length(); i++) {
+                char c = static_cast<char>(pColumn->getName()[i]);
+                invalidColumnType += c;
+            }
+            invalidColumnType += " " + cInvalidTypeSuffix;;
+                
+            throw string(invalidColumnType);
         } else {
-            throw string(cInvalidColumnType);
+            string invalidColumnType = cInvalidColumnPrefix + " ";
+            for(int i = 0; i < pColumn->getName().length(); i++) {
+                char c = static_cast<char>(pColumn->getName()[i]);
+                invalidColumnType += c;
+            }
+            invalidColumnType += " " + cInvalidTypeSuffix;;
+            
+            throw string(invalidColumnType);
         }
     }
     
@@ -83,6 +131,10 @@ public:
         return normalizedNumberVector;
     }
     float getNormalizedNumber(Column* pColumn, float number, bool limit = false) {
+        if(isnan(number)) {
+            return number;    
+        }
+        
         Column::COLUMN_TYPE type = pColumn->getColumnType();
         if(type == Column::NUMERICAL) {
             NumberColumn* pNumberColumn = dynamic_cast<NumberColumn*>(pColumn);
@@ -97,12 +149,25 @@ public:
                 }
             }
             
-            float normalizedNumber;
+            float normalizedNumber = 0;
             if(scaleType == Column::LINEAR) {
                 if(pNumberColumn->getMax() - pNumberColumn->getMin() > 0) {
                     normalizedNumber = (number - pNumberColumn->getMin()) / (pNumberColumn->getMax() - pNumberColumn->getMin());
                 } else {
-                    normalizedNumber = pNumberColumn->getMax();
+                    if(pNumberColumn->getMax() > 0) {
+                        //normalizedNumber = pNumberColumn->getMax();
+                        normalizedNumber = 1;
+                    }
+                }
+            }else if(scaleType == NumberColumn::LOGARITHMIC) {
+                if(pNumberColumn->getMax() - pNumberColumn->getMin() > 0) {
+                    normalizedNumber = log(number - pNumberColumn->getMin() + 1) /
+                        log(pNumberColumn->getMax() - pNumberColumn->getMin() + 1);
+                } else {
+                    if(pNumberColumn->getMax() > 0) {
+                        //normalizedNumber = pNumberColumn->getMax();
+                        normalizedNumber = 1;
+                    }
                 }
             } else {
                 throw cInvalidScaleType;
@@ -113,6 +178,10 @@ public:
         }
     }
     float getDenormalizedNumber(Column* pColumn, float number) {
+        if(isnan(number)) {
+            return number;
+        }
+        
         Column::COLUMN_TYPE type = pColumn->getColumnType();
         if(type == Column::NUMERICAL) {
             NumberColumn* pNumberColumn = dynamic_cast<NumberColumn*>(pColumn);
@@ -121,8 +190,10 @@ public:
             float denormalizedNumber;
             if(scaleType == Column::LINEAR) {
                 denormalizedNumber = pNumberColumn->getMin() + (pNumberColumn->getMax() - pNumberColumn->getMin()) * number;
+            } else if(scaleType == NumberColumn::LOGARITHMIC) {
+                denormalizedNumber = pNumberColumn->getMin() - 1 + exp(number * log(pNumberColumn->getMax() - pNumberColumn->getMin() + 1));
             } else {
-            throw cInvalidScaleType;
+                throw cInvalidScaleType;
             }
             return denormalizedNumber;
         } else {
